@@ -4,7 +4,7 @@ package POE::Component::Client::Asterisk::Manager;
 ### POE::Component::Client::Asterisk::Manager
 ### David Davis (xantus@cpan.org)
 ###
-### Copyright (c) 2003-2004 David Davis and Teknikill.  All Rights
+### Copyright (c) 2003-2005 David Davis and Teknikill.  All Rights
 ### Reserved. This module is free software; you can redistribute it
 ### and/or modify it under the same terms as Perl itself.
 ######################################################################
@@ -12,7 +12,7 @@ package POE::Component::Client::Asterisk::Manager;
 use strict;
 use warnings;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use Carp qw(croak);
 use POE qw( Component::Client::TCP );
@@ -65,7 +65,7 @@ sub _start {
 	$kernel->alias_set($params{Alias});
 
 	# watch for SIGINT
-	$kernel->sig('INT', 'signals');
+#	$kernel->sig('INT', 'signals');
 
 	$heap->{client} = POE::Component::Client::TCP->new(
 		RemoteAddress => $params{RemoteHost},
@@ -73,24 +73,29 @@ sub _start {
 		# no longer a seperate package - see below
 		Filter     => "POE::Filter::Asterisk::Manager",
 		Alias => "$params{Alias}_client",
+		Args => [ \%params ],
+		Started => sub {
+			$_[HEAP]->{params} = $_[ARG0];
+		},
 		Disconnected => sub {
-			$_[KERNEL]->delay(reconnect => $params{reconnect_time});
+			$_[KERNEL]->delay(reconnect => $_[HEAP]->{params}->{reconnect_time});
 		},
 		Connected => sub {
 			my $heap = $_[HEAP];
-			DEBUG && print STDERR "connected to $params{RemoteHost}:$params{RemotePort} ...\n";
+			DEBUG && print STDERR sprintf("connected to %s:%s\n",$heap->{params}->{RemoteHost},$heap->{params}->{RemotePort});
 			$heap->{_connected} = 0;
 			$heap->{_logged_in} = 0;
 			$heap->{_auth_stage} = 0;
 			$_[KERNEL]->delay( recv_timeout => 5 );
 		},
 		ConnectError => sub {
-			DEBUG && print STDERR "could not connect to $params{RemoteHost}:$params{RemotePort}, reconnecting in $params{reconnect_time} seconds...\n";
-			$_[KERNEL]->delay(reconnect => $params{reconnect_time});
+			my $heap = $_[HEAP];
+			DEBUG && print STDERR sprintf("could not connect to %s:%s, reconnecting in %s seconds...\n"
+				,$heap->{params}->{RemoteHost},$heap->{params}->{RemotePort},$heap->{params}->{reconnect_time});
+			$_[KERNEL]->delay(reconnect => $heap->{params}->{reconnect_time});
 		},
-
 		ServerInput => sub {
-			my ( $kernel, $heap, $input ) = @_[ KERNEL, HEAP, ARG0 ];
+			my ( $kernel, $heap, $input ) = @_[KERNEL, HEAP, ARG0];
 
 			DEBUG && do {
 				require Data::Dumper;
@@ -98,7 +103,7 @@ sub _start {
 			};
 
 			if ($heap->{_logged_in} == 0 && $heap->{_connected} == 0) {
-				$_[KERNEL]->delay( recv_timeout => 5 );
+				$kernel->delay( recv_timeout => 5 );
 				if ($input->{acm_version}) {
 					$heap->{_version} = $input->{acm_version};
 					$heap->{_connected} = 1;
@@ -116,24 +121,25 @@ sub _start {
 
 		InlineStates => {
 			_put => sub {
-				if ($_[HEAP]->{server}) {
-					$_[HEAP]->{server}->put($_[ARG0]);
+				my $heap = $_[HEAP];
+				if ($heap->{server}) {
+					$heap->{server}->put($_[ARG0]);
 				} else {
-					if ($_[HEAP]->{requeue_posts}) {
-						push(@{$_[HEAP]->{queued}},$_[ARG0]);
+					if ($heap->{requeue_posts}) {
+						push(@{$heap->{queued}},$_[ARG0]);
 					} else {
 						print STDERR "cannot send when not connected! -ignored-\n";
 					}
 				}
 			},
 			login_complete => sub {
-				my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
+				my ( $kernel, $heap ) = @_[KERNEL, HEAP];
 				DEBUG && print STDERR "logged in and ready to process events\n";
 				# call the _connected state
 				$kernel->yield("_connected" => splice(@_,ARG0));
 			},
 			recv_timeout => sub {
-				my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
+				my ( $kernel, $heap ) = @_[KERNEL, HEAP];
 				unless ($heap->{_connected} == 1) {
 					print STDERR "Timeout waiting for response\n";
 					$heap->{_connected} = 0;
@@ -142,7 +148,7 @@ sub _start {
 				}
 			},
 			login => sub {
-				my ($kernel, $heap, $input) = @_[KERNEL,HEAP,ARG0];
+				my ($kernel, $heap, $input) = @_[KERNEL, HEAP, ARG0];
 				if ($heap->{_logged_in} == 1) {
 					# shouldn't get here
 					DEBUG && print STDERR "Login called when already logged in\n";
@@ -159,16 +165,17 @@ sub _start {
 						return;
 					}
 					if ($input->{Challenge}) {
-						my $digest = Digest::MD5::md5_hex("$input->{Challenge}$params{Password}");
-						$heap->{server}->put({'Action' => 'Login', 'AuthType' => 'MD5', 'Username' => $params{Username}, 'Key' => $digest });
+						my $digest = Digest::MD5::md5_hex("$input->{Challenge}$heap->{params}->{Password}");
+						$heap->{server}->put({'Action' => 'Login', 'AuthType' => 'MD5', 'Username' => $heap->{params}->{Username}, 'Key' => $digest });
 						$heap->{_auth_stage} = 2;
 					}
 				} elsif ($heap->{_auth_stage} == 2) {
 					if ($input->{Message} && lc($input->{Message}) eq 'authentication accepted') {
 						delete $heap->{_auth_stage};
 						$heap->{_logged_in} = 1;
-						foreach my $k (keys %{$params{inline_states}}) {
-							$kernel->state( "$k" => $params{inline_states}{$k} );
+						# I remembered inline_states not working (above), so i put this in
+						foreach my $k (keys %{$heap->{params}->{inline_states}}) {
+							$kernel->state($k => $heap->{params}->{inline_states}{$k});
 						}
 						if (ref($heap->{queued}) eq 'ARRAY') {
 							foreach my $a (@{$heap->{queued}}) {
@@ -181,16 +188,16 @@ sub _start {
 				}
 			},
 			callbacks => sub {
-				my ($kernel, $heap, $session, $input) = @_[KERNEL,HEAP,SESSION,ARG0];
+				my ($kernel, $heap, $session, $input) = @_[KERNEL, HEAP, SESSION, ARG0];
 				# TODO this stuff needs some work
 				next unless (ref($input));
 				my $qual = 0;
-				foreach my $k (keys %{$params{CallBacks}}) {
+				foreach my $k (keys %{$heap->{params}->{CallBacks}}) {
 					my $match = 0;
-					if (ref($params{CallBacks}{$k}) eq 'HASH') {
-						foreach my $c (keys %{$params{CallBacks}{$k}}) {
+					if (ref($heap->{params}->{CallBacks}{$k}) eq 'HASH') {
+						foreach my $c (keys %{$heap->{params}->{CallBacks}{$k}}) {
 							last if ($match == 1);
-							if (exists($input->{$c}) && $params{CallBacks}{$k}{$c} eq $input->{$c}) {
+							if (exists($input->{$c}) && $heap->{params}->{CallBacks}{$k}{$c} eq $input->{$c}) {
 								$match = 2;
 								$qual++;
 							} else {
@@ -203,7 +210,7 @@ sub _start {
 							DEBUG && print STDERR "callback $k is good\n";
 							$kernel->yield($k => $input);
 						}
-					} elsif ($params{CallBacks}{$k} eq ':all' || $params{CallBacks}{$k} eq 'default') {
+					} elsif ($heap->{params}->{CallBacks}{$k} eq ':all' || $heap->{params}->{CallBacks}{$k} eq 'default') {
 						$kernel->yield($k => $input);
 					} else {
 						print STDERR "Incorrectly written callback $k\n";
@@ -294,7 +301,7 @@ sub get_one {
 	return [] if ($self->{finish});
 
 
-	if ($self->{buffer} =~ s#^Asterisk Call Manager/(\d+\.\d+)$self->{crlf}##is) {
+	if ($self->{buffer} =~ s#^(?:Asterisk|Aefirion) Call Manager/(\d+\.\d+)$self->{crlf}##is) {
 		return [{ acm_version => $1 }];
 	}
 
@@ -359,7 +366,7 @@ __END__
 
 =head1 NAME
 
-POE::Component::Client::Asterisk::Manager - Event-based Asterisk Manager Client
+POE::Component::Client::Asterisk::Manager - Event-based Asterisk / Aefirion Manager Client
 
 =head1 SYNOPSIS
 
@@ -397,7 +404,7 @@ POE::Component::Client::Asterisk::Manager - Event-based Asterisk Manager Client
 =head1 DESCRIPTION
 
 POE::Component::Client::Asterisk::Manager is an event driven Asterisk manager
-client
+client.  This module should also work with Aefirion.
 
 =head1 METHODS
 
@@ -425,23 +432,25 @@ You can specify a catch all event like this:
 
 	catch_all => ':all'
 
-Note: This was changed from 'default' to ':all' in an effort to make it more
-clear.  'default' will also work.
+Note: This was changed from 'default' to ':all' in an effort to make it 
+more clear.  'default' will also work.
 
 =head1 BUGS
 
-None known. Please report them to the author.
+Please report them via RT:
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=POE%3A%3AComponent%3A%3AClient%3A%3AAsterisk%3A%3AManager>
 
 =head1 EXAMPLES
 
 There are a few examples in the examples directory that can get you going.
 
-=head1 AUTHORS
+=head1 AUTHOR
 
 David Davis, E<lt>xantus@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
-perl(1)
+perl(1), POE::Filter::Asterisk::Manager, L<http://aefirion.org/>, L<http://asterisk.org/>,
+L<http://teknikill.net/>
 
 =cut
